@@ -4,7 +4,7 @@ class.fontMetric = "org/lysis/reporters/graphicaldevices/fonts/FontMetric"
 class.html4r.DataTable = "org/lysis/reporters/html4r/elements/DataTable"
 class.html4r.POT = "org/lysis/reporters/html4r/elements/POT"
 class.html4r.RScript = "org/lysis/reporters/html4r/elements/RScript"
-class.html4r.ImagesList = "org/lysis/reporters/html4r/elements/SetOfPlots"
+class.html4r.ImagesList = "org/lysis/reporters/html4r/elements/ImagesList"
 class.html4r.POTsList = "org/lysis/reporters/html4r/elements/SetOfPot"
 class.html4r.RaphaelList = "org/lysis/reporters/html4r/elements/SetOfRaphaelPlots"
 class.html4r.Title = "org/lysis/reporters/html4r/elements/Title"
