@@ -26,53 +26,51 @@
 
 #define R_USE_PROTOTYPES 1
 
-
 #include "datastruct.h"
-#include "utils.h"
+#include "colors.h"
+#include "dml_utils.h"
 #include "common.h"
 #include "DOCX.h"
+#include "utils.h"
 
 static char docx_elt_tag_start[] = "<wps:wsp>";
 static char docx_elt_tag_end[] = "</wps:wsp>";
 static char docx_lock_properties[] = "<wps:cNvSpPr><a:spLocks noSelect=\"1\" noResize=\"1\" noEditPoints=\"1\" noTextEdit=\"1\" noMove=\"1\" noRot=\"1\" noChangeShapeType=\"1\"/></wps:cNvSpPr><wps:nvPr />";
 static char docx_unlock_properties[] = "<wps:cNvSpPr /><wps:nvPr />";
 
-
 void DOCX_setRunProperties(pDevDesc dev, R_GE_gcontext *gc, double fontsize){
 	DOCDesc *pd = (DOCDesc *) dev->deviceSpecific;
 	int alpha =  (int) ((255-R_ALPHA(gc->col))/255.0 * 100000);//n importe quoi
 	int fontface = gc->fontface;
-	fprintf(pd->dmlFilePointer, "<w:rPr>" );
+	fputs("<w:rPr>", pd->dmlFilePointer );
 	fprintf(pd->dmlFilePointer, "<w:rFonts w:ascii=\"%s\" w:hAnsi=\"%s\" w:cs=\"%s\" />", pd->fi->fontname, pd->fi->fontname, pd->fi->fontname );
 	if ( fontface == 2 || fontface == 4 ) {
-		fprintf(pd->dmlFilePointer, "<w:b />");
+		fputs("<w:b />", pd->dmlFilePointer);
 	}
 	if (fontface == 3 || fontface == 4) {
-		fprintf(pd->dmlFilePointer, "<w:i />");
+		fputs( "<w:i />", pd->dmlFilePointer );
 	}
 	fprintf(pd->dmlFilePointer, "<w:color w:val=\"%s\" />", RGBHexValue(gc->col) );
 	fprintf(pd->dmlFilePointer, "<w:sz w:val=\"%.0f\" />", fontsize * 2 );
 	fprintf(pd->dmlFilePointer, "<w:szCs w:val=\"%.0f\" />", fontsize * 2 );
 
 	if (alpha > 0) {
-		fprintf(pd->dmlFilePointer, "<w14:textFill>" );
-		fprintf(pd->dmlFilePointer, "<w14:solidFill>" );
+		fputs( "<w14:textFill>", pd->dmlFilePointer );
+		fputs( "<w14:solidFill>", pd->dmlFilePointer );
 		fprintf(pd->dmlFilePointer, "<w14:srgbClr w14:val=\"%s\">", RGBHexValue(gc->col) );
 		fprintf(pd->dmlFilePointer, "<w14:alpha w14:val=\"%d\" />", alpha);
-		fprintf(pd->dmlFilePointer, "</w14:srgbClr>");
-		fprintf(pd->dmlFilePointer, "</w14:solidFill>" );
-		fprintf(pd->dmlFilePointer, "</w14:textFill>" );
+		fputs( "</w14:srgbClr>", pd->dmlFilePointer);
+		fputs( "</w14:solidFill>", pd->dmlFilePointer );
+		fputs( "</w14:textFill>", pd->dmlFilePointer );
 	}
 
-
-	fprintf(pd->dmlFilePointer, "</w:rPr>" );
-
+	fputs( "</w:rPr>", pd->dmlFilePointer );
 }
 
 
 static Rboolean DOCXDeviceDriver(pDevDesc dev, const char* filename, double* width,
 		double* height, double* offx, double* offy, double ps, int nbplots,
-		const char* fontname, SEXP env) {
+		const char* fontname, int id_init_value, int editable) {
 
 
 	DOCDesc *rd;
@@ -86,7 +84,7 @@ static Rboolean DOCXDeviceDriver(pDevDesc dev, const char* filename, double* wid
 
 	rd->filename = strdup(filename);
 	rd->fontname = strdup(fontname);
-	rd->id = 0;
+	rd->id = id_init_value;
 	rd->pageNumber = 0;
 	rd->offx = offx[0];
 	rd->offy = offy[0];
@@ -99,7 +97,7 @@ static Rboolean DOCXDeviceDriver(pDevDesc dev, const char* filename, double* wid
 	rd->height = height;
 	rd->fontface = 1;
 	rd->fontsize = (int) ps;
-	rd->env=env;
+	//rd->env=env;
 
 
 	//
@@ -160,14 +158,14 @@ static Rboolean DOCXDeviceDriver(pDevDesc dev, const char* filename, double* wid
 	dev->haveTransparency = 2;
 	dev->haveTransparentBg = 3;
 
-	rd->editable = getEditable(dev);
+	rd->editable = editable;
 
 	return (Rboolean) TRUE;
 }
 
 
 void GE_DOCXDevice(const char* filename, double* width, double* height, double* offx,
-		double* offy, double ps, int nbplots, const char* fontfamily, SEXP env) {
+		double* offy, double ps, int nbplots, const char* fontfamily, int id_init_value, int editable) {
 	pDevDesc dev = NULL;
 	pGEDevDesc dd;
 	R_GE_checkVersionOrDie (R_GE_version);
@@ -176,7 +174,7 @@ void GE_DOCXDevice(const char* filename, double* width, double* height, double*
 	if (!(dev = (pDevDesc) calloc(1, sizeof(DevDesc))))
 		Rf_error("unable to start DOCX device");
 	if (!DOCXDeviceDriver(dev, filename, width, height, offx, offy, ps, nbplots,
-			fontfamily, env)) {
+			fontfamily, id_init_value, editable)) {
 		free(dev);
 		Rf_error("unable to start DOCX device");
 	}
@@ -187,34 +185,32 @@ void GE_DOCXDevice(const char* filename, double* width, double* height, double*
 }
 
 static void DOCX_activate(pDevDesc dev) {
-
 }
 
-static void DOCX_Circle(double x, double y, double r, const pGEcontext gc,
-		pDevDesc dev) {
+static void DOCX_Circle(double x, double y, double r, const pGEcontext gc, pDevDesc dev) {
 	DOCDesc *pd = (DOCDesc *) dev->deviceSpecific;
-	int idx = get_idx(dev);
+	int idx = get_and_increment_idx(dev);
+
+	fputs( docx_elt_tag_start, pd->dmlFilePointer );
 
-	fprintf(pd->dmlFilePointer, docx_elt_tag_start);
 	if( pd->editable > 0 )
 		fprintf(pd->dmlFilePointer,	"<wps:cNvPr id=\"%d\" name=\"Point %d\" />%s", idx,	idx, docx_unlock_properties);
 	else fprintf(pd->dmlFilePointer,	"<wps:cNvPr id=\"%d\" name=\"Point %d\" />%s", idx,	idx, docx_lock_properties);
-	fprintf(pd->dmlFilePointer, "<wps:spPr>");
-	fprintf(pd->dmlFilePointer, "<a:xfrm>");
+	fputs( "<wps:spPr>", pd->dmlFilePointer );
+	fputs( "<a:xfrm>", pd->dmlFilePointer);
 	fprintf(pd->dmlFilePointer, "<a:off x=\"%.0f\" y=\"%.0f\"/>",
 			p2e_(pd->offx + x - r), p2e_(pd->offy + y - r));
 	fprintf(pd->dmlFilePointer, "<a:ext cx=\"%.0f\" cy=\"%.0f\"/>", p2e_(r * 2),
 			p2e_(r * 2));
-	fprintf(pd->dmlFilePointer, "</a:xfrm>");
-	fprintf(pd->dmlFilePointer,
-			"<a:prstGeom prst=\"ellipse\"><a:avLst /></a:prstGeom>");
-	SetFillColor(dev, gc);
-	SetLineSpec(dev, gc);
-	fprintf(pd->dmlFilePointer, "</wps:spPr>");
+	fputs( "</a:xfrm>", pd->dmlFilePointer);
+	fputs( "<a:prstGeom prst=\"ellipse\"><a:avLst /></a:prstGeom>", pd->dmlFilePointer);
 
-	fprintf(pd->dmlFilePointer, "<wps:bodyPr />");
-	fprintf(pd->dmlFilePointer, docx_elt_tag_end);
-	fprintf(pd->dmlFilePointer, "\n");
+	DML_SetFillColor(dev, gc);
+	DML_SetLineSpec(dev, gc);
+	fputs("</wps:spPr>", pd->dmlFilePointer );
+
+	fputs("<wps:bodyPr />", pd->dmlFilePointer );
+	fputs(docx_elt_tag_end, pd->dmlFilePointer );
 	fflush(pd->dmlFilePointer);
 }
 
@@ -238,47 +234,43 @@ static void DOCX_Line(double x1, double y1, double x2, double y2,
 		miny = y2;
 	}
 
-//	Rprintf("line x{from %.3f to %.3f} y{from %.3f to %.3f}\n", x1, x2, y1, y2);
 	DOCDesc *pd = (DOCDesc *) dev->deviceSpecific;
-	int idx = get_idx(dev);
+	int idx = get_and_increment_idx(dev);
 
-	fprintf(pd->dmlFilePointer, docx_elt_tag_start);
+	fputs(docx_elt_tag_start, pd->dmlFilePointer );
+	
 	if( pd->editable > 0 )
 		fprintf(pd->dmlFilePointer,
 			"<wps:cNvPr id=\"%d\" name=\"Line %d\" />%s", idx,	idx, docx_unlock_properties);
 	else fprintf(pd->dmlFilePointer,
 			"<wps:cNvPr id=\"%d\" name=\"Line %d\" />%s", idx,	idx, docx_lock_properties);
-	fprintf(pd->dmlFilePointer, "<wps:spPr>");
-		fprintf(pd->dmlFilePointer, "<a:xfrm>");
+	fputs("<wps:spPr>", pd->dmlFilePointer );
+		fputs("<a:xfrm>", pd->dmlFilePointer );//fprintf(pd->dmlFilePointer, "<a:xfrm>");
 			fprintf(pd->dmlFilePointer, "<a:off x=\"%.0f\" y=\"%.0f\"/>"
 					, p2e_(pd->offx + minx), p2e_(pd->offy + miny));
 			fprintf(pd->dmlFilePointer, "<a:ext cx=\"%.0f\" cy=\"%.0f\"/>"
 					, p2e_(maxx-minx), p2e_(maxy-miny));
 
-//	Rprintf("\toff {%.0f ; %.0f} ext{%.0f ; %.0f}\n", p2e_(pd->offx + x1), p2e_(pd->offy + y1), p2e_(maxx-minx), p2e_(maxy-miny));
-
-		fprintf(pd->dmlFilePointer, "</a:xfrm>");
-		fprintf(pd->dmlFilePointer, "<a:custGeom>");
-		fprintf(pd->dmlFilePointer, "<a:pathLst>");
+		fputs("</a:xfrm>", pd->dmlFilePointer );
+		fputs("<a:custGeom>", pd->dmlFilePointer );
+		fputs("<a:pathLst>", pd->dmlFilePointer );
 			fprintf(pd->dmlFilePointer, "<a:path w=\"%.0f\" h=\"%.0f\">", p2e_(maxx-minx), p2e_(maxy-miny));
 
 
 				fprintf(pd->dmlFilePointer,
 						"<a:moveTo><a:pt x=\"%.0f\" y=\"%.0f\" /></a:moveTo>", p2e_(x1 - minx), p2e_(y1 - miny));
 				fprintf(pd->dmlFilePointer,
-						"<a:lnTo><a:pt x=\"%.0f\" y=\"%.0f\" /></a:lnTo>", p2e_(x2 - minx),
-						p2e_(y2 - miny));
-			fprintf(pd->dmlFilePointer, "</a:path>");
-		fprintf(pd->dmlFilePointer, "</a:pathLst>");
+						"<a:lnTo><a:pt x=\"%.0f\" y=\"%.0f\" /></a:lnTo>", p2e_(x2 - minx), p2e_(y2 - miny));
+			fputs("</a:path>", pd->dmlFilePointer );
+		fputs("</a:pathLst>", pd->dmlFilePointer );
 
-	fprintf(pd->dmlFilePointer, "</a:custGeom><a:noFill />");
-	SetLineSpec(dev, gc);
-	fprintf(pd->dmlFilePointer, "</wps:spPr>");
+	fputs("</a:custGeom><a:noFill />", pd->dmlFilePointer );
+	DML_SetLineSpec(dev, gc);
+	fputs("</wps:spPr>", pd->dmlFilePointer );
 
-	fprintf(pd->dmlFilePointer,
-			"<wps:bodyPr />");
-	fprintf(pd->dmlFilePointer, docx_elt_tag_end);
-	fprintf(pd->dmlFilePointer, "\n");
+	fputs("<wps:bodyPr />", pd->dmlFilePointer );
+	fputs(docx_elt_tag_end, pd->dmlFilePointer );
+	//fprintf(pd->dmlFilePointer, "\n");
 
 	fflush(pd->dmlFilePointer);
 
@@ -287,10 +279,9 @@ static void DOCX_Line(double x1, double y1, double x2, double y2,
 
 static void DOCX_Polyline(int n, double *x, double *y, const pGEcontext gc,
 		pDevDesc dev) {
-//	Rprintf("DOCX_Polyline\n");
 
 	DOCDesc *pd = (DOCDesc *) dev->deviceSpecific;
-	int idx = get_idx(dev);
+	int idx = get_and_increment_idx(dev);
 	int i;
 	double maxx = 0, maxy = 0;
 	for (i = 0; i < n; i++) {
@@ -308,23 +299,23 @@ static void DOCX_Polyline(int n, double *x, double *y, const pGEcontext gc,
 			miny = y[i];
 	}
 //
-	fprintf(pd->dmlFilePointer, docx_elt_tag_start);
+	fputs(docx_elt_tag_start, pd->dmlFilePointer );
 	if( pd->editable < 1 )
 		fprintf(pd->dmlFilePointer,
 			"<wps:cNvPr id=\"%d\" name=\"Polyline %d\" />%s", idx,	idx, docx_lock_properties);
 	else fprintf(pd->dmlFilePointer,
 			"<wps:cNvPr id=\"%d\" name=\"Polyline %d\" />%s", idx,	idx, docx_unlock_properties);
 
-	fprintf(pd->dmlFilePointer, "<wps:spPr>");
-	fprintf(pd->dmlFilePointer, "<a:xfrm>");
+	fputs("<wps:spPr>", pd->dmlFilePointer );
+	fputs("<a:xfrm>", pd->dmlFilePointer );
 	fprintf(pd->dmlFilePointer, "<a:off x=\"%.0f\" y=\"%.0f\"/>",
 			p2e_(pd->offx + minx), p2e_(pd->offy + miny));
 	fprintf(pd->dmlFilePointer, "<a:ext cx=\"%.0f\" cy=\"%.0f\"/>",
 			p2e_(maxx - minx), p2e_(maxy - miny));
-	fprintf(pd->dmlFilePointer, "</a:xfrm>");
-	fprintf(pd->dmlFilePointer, "<a:custGeom>");
+	fputs("</a:xfrm>", pd->dmlFilePointer );
+	fputs("<a:custGeom>", pd->dmlFilePointer );
 
-	fprintf(pd->dmlFilePointer, "<a:pathLst>");
+	fputs("<a:pathLst>", pd->dmlFilePointer );
 	fprintf(pd->dmlFilePointer, "<a:path w=\"%.0f\" h=\"%.0f\">", p2e_(maxx-minx), p2e_(maxy-miny));
 
 
@@ -339,27 +330,25 @@ static void DOCX_Polyline(int n, double *x, double *y, const pGEcontext gc,
 	}
 	//fprintf(pd->dmlFilePointer, "<a:close/>");
 
-	fprintf(pd->dmlFilePointer, "</a:path>");
-	fprintf(pd->dmlFilePointer, "</a:pathLst>");
-	fprintf(pd->dmlFilePointer, "</a:custGeom>");
-	//SetFillColor(dev, gc);
-	SetLineSpec(dev, gc);
-	fprintf(pd->dmlFilePointer, "</wps:spPr>");
+	fputs( "</a:path>", pd->dmlFilePointer );
+	fputs( "</a:pathLst>", pd->dmlFilePointer );
+	fputs( "</a:custGeom>", pd->dmlFilePointer );
 
-	fprintf(pd->dmlFilePointer, "<wps:bodyPr />");
-	fprintf(pd->dmlFilePointer, docx_elt_tag_end);
-	fprintf(pd->dmlFilePointer, "\n");
+	DML_SetLineSpec(dev, gc);
+	fputs( "</wps:spPr>", pd->dmlFilePointer );
+
+	fputs( "<wps:bodyPr />", pd->dmlFilePointer );
+	fputs(docx_elt_tag_end, pd->dmlFilePointer );
+//	fprintf(pd->dmlFilePointer, "\n");
 	fflush(pd->dmlFilePointer);
 
-	//return;
-
 }
 
 static void DOCX_Polygon(int n, double *x, double *y, const pGEcontext gc,
 		pDevDesc dev) {
 
 	DOCDesc *pd = (DOCDesc *) dev->deviceSpecific;
-	int idx = get_idx(dev);
+	int idx = get_and_increment_idx(dev);
 	int i;
 	double maxx = 0, maxy = 0;
 	for (i = 0; i < n; i++) {
@@ -377,22 +366,24 @@ static void DOCX_Polygon(int n, double *x, double *y, const pGEcontext gc,
 			miny = y[i];
 	}
 //
-	fprintf(pd->dmlFilePointer, docx_elt_tag_start);
+	fputs(docx_elt_tag_start, pd->dmlFilePointer );
 	if( pd->editable < 1 )
 		fprintf(pd->dmlFilePointer,
 			"<wps:cNvPr id=\"%d\" name=\"Polygon %d\" />%s", idx,	idx, docx_lock_properties);
 	else fprintf(pd->dmlFilePointer,
 			"<wps:cNvPr id=\"%d\" name=\"Polygon %d\" />%s", idx,	idx, docx_unlock_properties);
 
-	fprintf(pd->dmlFilePointer, "<wps:spPr>");
-	fprintf(pd->dmlFilePointer, "<a:xfrm>");
+	fputs( "<wps:spPr>", pd->dmlFilePointer );
+	fputs( "<a:xfrm>", pd->dmlFilePointer );
+
 	fprintf(pd->dmlFilePointer, "<a:off x=\"%.0f\" y=\"%.0f\"/>",
 			p2e_(pd->offx + minx), p2e_(pd->offy + miny));
 	fprintf(pd->dmlFilePointer, "<a:ext cx=\"%.0f\" cy=\"%.0f\"/>",
 			p2e_(maxx - minx), p2e_(maxy - miny));
-	fprintf(pd->dmlFilePointer, "</a:xfrm>");
-	fprintf(pd->dmlFilePointer, "<a:custGeom>");
-	fprintf(pd->dmlFilePointer, "<a:pathLst>");
+
+	fputs( "</a:xfrm>", pd->dmlFilePointer );
+	fputs( "<a:custGeom>", pd->dmlFilePointer );
+	fputs( "<a:pathLst>", pd->dmlFilePointer );
 	fprintf(pd->dmlFilePointer, "<a:path w=\"%.0f\" h=\"%.0f\">", p2e_(maxx-minx), p2e_(maxy-miny));
 	fprintf(pd->dmlFilePointer,
 			"<a:moveTo><a:pt x=\"%.0f\" y=\"%.0f\" /></a:moveTo>",
@@ -403,29 +394,27 @@ static void DOCX_Polygon(int n, double *x, double *y, const pGEcontext gc,
 				"<a:lnTo><a:pt x=\"%.0f\" y=\"%.0f\" /></a:lnTo>",
 				p2e_(x[i] - minx), p2e_(y[i] - miny));
 	}
-	fprintf(pd->dmlFilePointer, "<a:close/>");
+	fputs( "<a:close/>", pd->dmlFilePointer );
 
-	fprintf(pd->dmlFilePointer, "</a:path>");
-	fprintf(pd->dmlFilePointer, "</a:pathLst>");
-	fprintf(pd->dmlFilePointer, "</a:custGeom>");
-	SetFillColor(dev, gc);
-	SetLineSpec(dev, gc);
-	fprintf(pd->dmlFilePointer, "</wps:spPr>");
+	fputs( "</a:path>", pd->dmlFilePointer );
+	fputs( "</a:pathLst>", pd->dmlFilePointer );
+	fputs( "</a:custGeom>", pd->dmlFilePointer );
 
-	fprintf(pd->dmlFilePointer, "<wps:bodyPr />");
-	fprintf(pd->dmlFilePointer, docx_elt_tag_end);
-	fprintf(pd->dmlFilePointer, "\n");
+	DML_SetFillColor(dev, gc);
+	DML_SetLineSpec(dev, gc);
+	fputs( "</wps:spPr>", pd->dmlFilePointer );
+
+	fputs( "<wps:bodyPr />", pd->dmlFilePointer );
+	fputs(docx_elt_tag_end, pd->dmlFilePointer );
 	fflush(pd->dmlFilePointer);
 
-	//return;
-
 }
 
 static void DOCX_Rect(double x0, double y0, double x1, double y1,
 		const pGEcontext gc, pDevDesc dev) {
 	double tmp;
 	DOCDesc *pd = (DOCDesc *) dev->deviceSpecific;
-	int idx = get_idx(dev);
+	int idx = get_and_increment_idx(dev);
 
 	if (x0 >= x1) {
 		tmp = x0;
@@ -439,28 +428,28 @@ static void DOCX_Rect(double x0, double y0, double x1, double y1,
 		y1 = tmp;
 	}
 //
-	fprintf(pd->dmlFilePointer, docx_elt_tag_start);
+	fputs(docx_elt_tag_start, pd->dmlFilePointer );
 	if( pd->editable < 1 )
 		fprintf(pd->dmlFilePointer,
 			"<wps:cNvPr id=\"%d\" name=\"Rectangle %d\" />%s", idx,	idx, docx_lock_properties);
 	else fprintf(pd->dmlFilePointer,
 			"<wps:cNvPr id=\"%d\" name=\"Rectangle %d\" />%s", idx,	idx, docx_unlock_properties);
-	fprintf(pd->dmlFilePointer, "<wps:spPr>");
-	fprintf(pd->dmlFilePointer, "<a:xfrm>");
+	fputs("<wps:spPr>", pd->dmlFilePointer );
+	fputs("<a:xfrm>", pd->dmlFilePointer );
+
 	fprintf(pd->dmlFilePointer, "<a:off x=\"%.0f\" y=\"%.0f\"/>",
 			p2e_(pd->offx + x0), p2e_(pd->offy + y0));
 	fprintf(pd->dmlFilePointer, "<a:ext cx=\"%.0f\" cy=\"%.0f\"/>",
 			p2e_(x1 - x0), p2e_(y1 - y0));
-	fprintf(pd->dmlFilePointer, "</a:xfrm>");
-	fprintf(pd->dmlFilePointer,
-			"<a:prstGeom prst=\"rect\"><a:avLst /></a:prstGeom>");
-	SetFillColor(dev, gc);
-	SetLineSpec(dev, gc);
-	fprintf(pd->dmlFilePointer, "</wps:spPr>");
+	fputs("</a:xfrm>", pd->dmlFilePointer );
+	fputs("<a:prstGeom prst=\"rect\"><a:avLst /></a:prstGeom>", pd->dmlFilePointer );
+	DML_SetFillColor(dev, gc);
+	DML_SetLineSpec(dev, gc);
+	fputs("</wps:spPr>", pd->dmlFilePointer );
 
-	fprintf(pd->dmlFilePointer, "<wps:bodyPr />");
-	fprintf(pd->dmlFilePointer, docx_elt_tag_end);
-	fprintf(pd->dmlFilePointer, "\n");
+	fputs("<wps:bodyPr />", pd->dmlFilePointer );
+	fputs(docx_elt_tag_end, pd->dmlFilePointer );
+//	fprintf(pd->dmlFilePointer, "\n");
 	fflush(pd->dmlFilePointer);
 
 	//return;
@@ -473,7 +462,7 @@ static void DOCX_Text(double x, double y, const char *str, double rot,
 
 	DOCDesc *pd = (DOCDesc *) dev->deviceSpecific;
 	double pi = 3.141592653589793115997963468544185161590576171875;
-	int idx = get_idx(dev);
+	int idx = get_and_increment_idx(dev);
 
 	double w = DOCX_StrWidth(str, gc, dev);
 	if( strlen(str) < 3 ) w+= 1 * w / strlen(str);
@@ -502,43 +491,47 @@ static void DOCX_Text(double x, double y, const char *str, double rot,
 	if( rot < 0.05 || rot > 0.05 ) rot = -rot;
 	else rot = 0.0;
 
-	fprintf(pd->dmlFilePointer, docx_elt_tag_start);
+	fputs(docx_elt_tag_start, pd->dmlFilePointer );
 	if( pd->editable < 1 )
 		fprintf(pd->dmlFilePointer, "<wps:cNvPr id=\"%d\" name=\"Text %d\" />%s", idx,	idx, docx_lock_properties);
 	else fprintf(pd->dmlFilePointer, "<wps:cNvPr id=\"%d\" name=\"Text %d\" />%s", idx,	idx, docx_unlock_properties);
-	fprintf(pd->dmlFilePointer, "<wps:spPr>");
+
+	fputs("<wps:spPr>", pd->dmlFilePointer );
+
 	fprintf(pd->dmlFilePointer, "<a:xfrm rot=\"%.0f\">", rot * 60000);
 	fprintf(pd->dmlFilePointer, "<a:off x=\"%.0f\" y=\"%.0f\"/>", p2e_(pd->offx + corrected_offx), p2e_(pd->offy + corrected_offy));
 	fprintf(pd->dmlFilePointer, "<a:ext cx=\"%.0f\" cy=\"%.0f\"/>", p2e_(w), p2e_(h));
-	fprintf(pd->dmlFilePointer, "</a:xfrm>");
-	fprintf(pd->dmlFilePointer, "<a:prstGeom prst=\"rect\"><a:avLst /></a:prstGeom>");
-	fprintf(pd->dmlFilePointer, "<a:noFill />");
-	fprintf(pd->dmlFilePointer, "</wps:spPr>");
-	fprintf(pd->dmlFilePointer, "<wps:txbx>");
-	fprintf(pd->dmlFilePointer, "<w:txbxContent>");
-	fprintf(pd->dmlFilePointer, "<w:p>");
-	fprintf(pd->dmlFilePointer, "<w:pPr>");
+	fputs("</a:xfrm>", pd->dmlFilePointer );
+	fputs("<a:prstGeom prst=\"rect\"><a:avLst /></a:prstGeom>", pd->dmlFilePointer );
+	fputs("<a:noFill />", pd->dmlFilePointer );
+	fputs("</wps:spPr>", pd->dmlFilePointer );
+	fputs("<wps:txbx>", pd->dmlFilePointer );
+	
+	fputs("<w:txbxContent>", pd->dmlFilePointer );
+	fputs("<w:p>", pd->dmlFilePointer );
+	fputs("<w:pPr>", pd->dmlFilePointer );
+
 	if (hadj < 0.25)
-		fprintf(pd->dmlFilePointer, "<w:jc w:val=\"left\" />");
+		fputs("<w:jc w:val=\"left\" />", pd->dmlFilePointer );
 	else if (hadj < 0.75)
-		fprintf(pd->dmlFilePointer, "<w:jc w:val=\"center\" />");
+		fputs("<w:jc w:val=\"center\" />", pd->dmlFilePointer );
 	else
-		fprintf(pd->dmlFilePointer, "<w:jc w:val=\"right\" />");
+		fputs("<w:jc w:val=\"right\" />", pd->dmlFilePointer );
 
 	fprintf(pd->dmlFilePointer, "<w:spacing w:after=\"0\" w:before=\"0\" w:line=\"%.0f\" w:lineRule=\"exact\" />", height*20);
 	DOCX_setRunProperties( dev, gc, fontsize);
-	fprintf(pd->dmlFilePointer, "</w:pPr>");
-	fprintf(pd->dmlFilePointer, "<w:r>");
+	fputs("</w:pPr>", pd->dmlFilePointer );
+	fputs("<w:r>", pd->dmlFilePointer );
+	
 	DOCX_setRunProperties( dev, gc, fontsize);
 	fprintf(pd->dmlFilePointer, "<w:t>%s</w:t></w:r></w:p>", str);
-	fprintf(pd->dmlFilePointer, "</w:txbxContent>");
-	fprintf(pd->dmlFilePointer, "</wps:txbx>");
-	fprintf(pd->dmlFilePointer, "<wps:bodyPr lIns=\"0\" tIns=\"0\" rIns=\"0\" bIns=\"0\"");
-	fprintf(pd->dmlFilePointer, " anchor=\"b\">");
-	fprintf(pd->dmlFilePointer, "<a:spAutoFit />");
-	fprintf(pd->dmlFilePointer, "</wps:bodyPr>");
-	fprintf(pd->dmlFilePointer, docx_elt_tag_end);
-	fprintf(pd->dmlFilePointer, "\n");
+	fputs("</w:txbxContent>", pd->dmlFilePointer );
+	fputs("</wps:txbx>", pd->dmlFilePointer );
+	fputs("<wps:bodyPr lIns=\"0\" tIns=\"0\" rIns=\"0\" bIns=\"0\" anchor=\"b\">", pd->dmlFilePointer );
+
+	fputs( "<a:spAutoFit /></wps:bodyPr>", pd->dmlFilePointer);
+	fputs(docx_elt_tag_end, pd->dmlFilePointer );
+//	fprintf(pd->dmlFilePointer, "\n");
 	fflush(pd->dmlFilePointer);
 
 	//return;
@@ -553,7 +546,7 @@ static void DOCX_NewPage(const pGEcontext gc, pDevDesc dev) {
 	int which = pd->pageNumber % pd->maxplot;
 	pd->pageNumber++;
 
-	update_start_id(dev);
+	//update_start_id(dev);
 	dev->right = pd->width[which];
 	dev->bottom = pd->height[which];
 	pd->offx = pd->x[which];
@@ -562,7 +555,7 @@ static void DOCX_NewPage(const pGEcontext gc, pDevDesc dev) {
 	pd->exty = pd->height[which];
 
 	char *str={0};
-	str = getFilename(pd->filename, pd->pageNumber);
+	str = get_dml_filename(pd->filename, pd->pageNumber);
 	pd->dmlFilePointer = (FILE *) fopen(str, "w");
 
 	if (pd->dmlFilePointer == NULL) {
@@ -573,7 +566,7 @@ static void DOCX_NewPage(const pGEcontext gc, pDevDesc dev) {
 }
 static void DOCX_Close(pDevDesc dev) {
 	DOCDesc *pd = (DOCDesc *) dev->deviceSpecific;
-	update_start_id(dev);
+	//update_start_id(dev);
 	closeFile(pd->dmlFilePointer);
 	free(pd);
 }
@@ -632,7 +625,7 @@ static void javaGDresize_(int dev) {
 
 SEXP R_DOCX_Device(SEXP filename
 		, SEXP width, SEXP height, SEXP offx,
-		SEXP offy, SEXP pointsize, SEXP fontfamily, SEXP env) {
+		SEXP offy, SEXP pointsize, SEXP fontfamily, SEXP start_id, SEXP is_editable ) {
 
 	double* w = REAL(width);
 	double* h = REAL(height);
@@ -642,9 +635,14 @@ SEXP R_DOCX_Device(SEXP filename
 	int nx = length(width);
 
 	double ps = asReal(pointsize);
-
+	int id_init_value = INTEGER(start_id)[0];
+	int editable = INTEGER(is_editable)[0];
 	BEGIN_SUSPEND_INTERRUPTS;
-	GE_DOCXDevice(CHAR(STRING_ELT(filename, 0)), w, h, x, y, ps, nx, CHAR(STRING_ELT(fontfamily, 0)), env);
+	GE_DOCXDevice(CHAR(STRING_ELT(filename, 0))
+			, w, h, x, y, ps, nx, CHAR(STRING_ELT(fontfamily, 0))
+			, id_init_value, editable
+			);
 	END_SUSPEND_INTERRUPTS;
 	return R_NilValue;
 }
+
