@@ -28,30 +28,14 @@
 #include <R_ext/GraphicsDevice.h>
 
 extern "C" {
+	void get_current_canvas_id(int *dn, int *res);
+	void get_current_element_id(int *dn, int *res);
 
-double p2e_(double x);
-double p2t(double x);
-int getFontface( int ff );
-double getFontSize(double cex, double fontsize, double lineheight);
+	void set_tracer_on(int *dn);
+	void set_tracer_off(int *dn);
+	void collect_id(int *dn, int *res);
 
-
-void closeFile( FILE *file);
-
-char* getFilename(char* filename, int index);
-char* getRaphaelFilename(char* filename, int index);
-char* getJSVariableName(char* filename, int index);
-char* getCanvasName( int index);
-void update_canvas_id(pDevDesc dev);
-char *RGBHexValue(unsigned int col);
-
-void updateFontInfo(pDevDesc dev, R_GE_gcontext *gc);
-void update_start_id(pDevDesc dev);
-int getEditable(pDevDesc dev);
-int get_idx(pDevDesc dev);
-void SetFillColor(pDevDesc dev, R_GE_gcontext *gc);
-void SetFontColor(pDevDesc dev, R_GE_gcontext *gc);
-void SetLineSpec(pDevDesc dev, R_GE_gcontext *gc);
-void RAPHAEL_SetLineSpec(pDevDesc dev, R_GE_gcontext *gc, int idx);
-void RAPHAEL_SetFillColor(pDevDesc dev, R_GE_gcontext *gc, int idx);
-void RAPHAEL_SetFontSpec(pDevDesc dev, R_GE_gcontext *gc, int idx);
+	void add_popup(int *dn, int *id, char **str, int *l);
+	void add_click(int *dn, int *id, char **str, int *l);
+	void add_dblclick(int *dn, int *id, char **str, int *l);
 }
